@@ -1,15 +1,16 @@
 /* jshint mocha: true */
+/* eslint-env node, mocha */
 
 /**
  * Module dependencies.
  */
 
-var ejs = require('..')
-  , fs = require('fs')
-  , read = fs.readFileSync
-  , assert = require('assert')
-  , path = require('path')
-  , LRU = require('lru-cache');
+var ejs = require('..');
+var fs = require('fs');
+var read = fs.readFileSync;
+var assert = require('assert');
+var path = require('path');
+var LRU = require('lru-cache');
 
 try {
   fs.mkdirSync(__dirname + '/tmp');
@@ -103,10 +104,28 @@ suite('ejs.compile(str, options)', function () {
     delete ejs.delimiter;
   });
 
+  test('support custom escape function', function () {
+    var customEscape;
+    var fn;
+    customEscape = function customEscape(str) {
+      return !str ? '' : str.toUpperCase();
+    };
+    fn = ejs.compile('HELLO <%= name %>', {escape: customEscape});
+    assert.equal(fn({name: 'world'}), 'HELLO WORLD');
+  });
+
+  test('strict mode works', function () {
+    assert.equal(ejs.render(fixture('strict.ejs'), {}, {strict: true}), 'true');
+  });
+
+});
+
+suite('client mode', function () {
+
   test('have a working client option', function () {
-    var fn
-      , str
-      , preFn;
+    var fn;
+    var str;
+    var preFn;
     fn = ejs.compile('<p><%= foo %></p>', {client: true});
     str = fn.toString();
     if (!process.env.running_under_istanbul) {
@@ -116,9 +135,9 @@ suite('ejs.compile(str, options)', function () {
   });
 
   test('support client mode without locals', function () {
-    var fn
-      , str
-      , preFn;
+    var fn;
+    var str;
+    var preFn;
     fn = ejs.compile('<p><%= "foo" %></p>', {client: true});
     str = fn.toString();
     if (!process.env.running_under_istanbul) {
@@ -129,27 +148,17 @@ suite('ejs.compile(str, options)', function () {
 
   test('not include rethrow() in client mode if compileDebug is false', function () {
     var fn = ejs.compile('<p><%= "foo" %></p>', {
-               client: true
-             , compileDebug: false
-             });
+      client: true,
+      compileDebug: false
+    });
     // There could be a `rethrow` in the function declaration
     assert((fn.toString().match(/rethrow/g) || []).length <= 1);
   });
 
-  test('support custom escape function', function () {
-    var customEscape
-      , fn;
-    customEscape = function customEscape(str) {
-      return !str ? '' : str.toUpperCase();
-    };
-    fn = ejs.compile('HELLO <%= name %>', {escape: customEscape});
-    assert.equal(fn({name: 'world'}), 'HELLO WORLD');
-  });
-
   test('support custom escape function in client mode', function () {
-    var customEscape
-      , fn
-      , str;
+    var customEscape;
+    var fn;
+    var str;
     customEscape = function customEscape(str) {
       return !str ? '' : str.toUpperCase();
     };
@@ -157,14 +166,31 @@ suite('ejs.compile(str, options)', function () {
     str = fn.toString();
     if (!process.env.running_under_istanbul) {
       eval('var preFn = ' + str);
-      assert.equal(preFn({name: 'world'}), 'HELLO WORLD');
+      assert.equal(preFn({name: 'world'}), 'HELLO WORLD'); // eslint-disable-line no-undef
     }
   });
 
-  test('strict mode works', function () {
-    assert.equal(ejs.render(fixture('strict.ejs'), {}, {strict: true}), 'true');
+  test('escape filename in errors in client mode', function () {
+    assert.throws(function () {
+      var fn = ejs.compile('<% throw new Error("whoops"); %>', {client: true, filename: '<script>'});
+      fn();
+    }, /Error: &lt;script&gt;/);
+  });
+});
+
+/* Old API -- remove when this shim goes away */
+suite('ejs.render(str, dataAndOpts)', function () {
+  test('render the template with data/opts passed together', function () {
+    assert.equal(ejs.render('<p><?= foo ?></p>', {foo: 'yay', delimiter: '?'}),
+        '<p>yay</p>');
   });
 
+  test('disallow unsafe opts passed along in data', function () {
+    assert.equal(ejs.render('<p><?= locals.foo ?></p>',
+        // localsName should not get reset because it's blacklisted
+        {_with: false, foo: 'yay', delimiter: '?', localsName: '_'}),
+        '<p>yay</p>');
+  });
 });
 
 suite('ejs.render(str, data, opts)', function () {
@@ -228,10 +254,10 @@ suite('ejs.render(str, data, opts)', function () {
   });
 
   test('support caching', function () {
-    var file = __dirname + '/tmp/render.ejs'
-      , options = {cache: true, filename: file}
-      , out = ejs.render('<p>Old</p>', {}, options)
-      , expected = '<p>Old</p>';
+    var file = __dirname + '/tmp/render.ejs';
+    var options = {cache: true, filename: file};
+    var out = ejs.render('<p>Old</p>', {}, options);
+    var expected = '<p>Old</p>';
     assert.equal(out, expected);
     // Assert no change, still in cache
     out = ejs.render('<p>New</p>', {}, options);
@@ -239,11 +265,11 @@ suite('ejs.render(str, data, opts)', function () {
   });
 
   test('support LRU caching', function () {
-    var oldCache = ejs.cache
-      , file = __dirname + '/tmp/render.ejs'
-      , options = {cache: true, filename: file}
-      , out
-      , expected = '<p>Old</p>';
+    var oldCache = ejs.cache;
+    var file = __dirname + '/tmp/render.ejs';
+    var options = {cache: true, filename: file};
+    var out;
+    var expected = '<p>Old</p>';
 
     // Switch to LRU
     ejs.cache = LRU();
@@ -259,10 +285,11 @@ suite('ejs.render(str, data, opts)', function () {
   });
 
   test('opts.context', function () {
-    var ctxt = {foo: 'FOO'}
-      , out = ejs.render('<%= this.foo %>', {}, {context: ctxt});
+    var ctxt = {foo: 'FOO'};
+    var out = ejs.render('<%= this.foo %>', {}, {context: ctxt});
     assert.equal(out, ctxt.foo);
   });
+
 });
 
 suite('ejs.renderFile(path, [data], [options], fn)', function () {
@@ -277,8 +304,8 @@ suite('ejs.renderFile(path, [data], [options], fn)', function () {
   });
 
   test('accept locals', function(done) {
-    var data =  {name: 'fonebone'}
-      , options = {delimiter: '$'};
+    var data =  {name: 'fonebone'};
+    var options = {delimiter: '$'};
     ejs.renderFile('test/fixtures/user.ejs', data, options, function(err, html) {
       if (err) {
         return done(err);
@@ -289,11 +316,10 @@ suite('ejs.renderFile(path, [data], [options], fn)', function () {
   });
 
   test('accept locals without using with() {}', function(done) {
-    var data =  {name: 'fonebone'}
-      , options = {delimiter: '$', _with: false}
-      , doneCount = 0;
-    ejs.renderFile('test/fixtures/user-no-with.ejs', data, options,
-                   function(err, html) {
+    var data =  {name: 'fonebone'};
+    var options = {delimiter: '$', _with: false};
+    var doneCount = 0;
+    ejs.renderFile('test/fixtures/user-no-with.ejs', data, options, function(err, html) {
       if (err) {
         if (doneCount === 2) {
           return;
@@ -323,9 +349,9 @@ suite('ejs.renderFile(path, [data], [options], fn)', function () {
   });
 
   test('not catch err thrown by callback', function(done) {
-    var data =  {name: 'fonebone'}
-      , options = {delimiter: '$'}
-      , counter = 0;
+    var data =  {name: 'fonebone'};
+    var options = {delimiter: '$'};
+    var counter = 0;
 
     var d = require('domain').create();
     d.on('error', function (err) {
@@ -340,8 +366,7 @@ suite('ejs.renderFile(path, [data], [options], fn)', function () {
       // domains. Have to make it async. Ticket closed because: "domains are
       // deprecated :D"
       process.nextTick(function () {
-        ejs.renderFile('test/fixtures/user.ejs', data, options,
-                       function(err) {
+        ejs.renderFile('test/fixtures/user.ejs', data, options, function(err) {
           counter++;
           if (err) {
             assert.notEqual(err.message, 'Exception in callback');
@@ -354,9 +379,9 @@ suite('ejs.renderFile(path, [data], [options], fn)', function () {
   });
 
   test('support caching', function (done) {
-    var expected = '<p>Old</p>'
-      , file = __dirname + '/tmp/renderFile.ejs'
-      , options = {cache: true};
+    var expected = '<p>Old</p>';
+    var file = __dirname + '/tmp/renderFile.ejs';
+    var options = {cache: true};
     fs.writeFileSync(file, '<p>Old</p>');
 
     ejs.renderFile(file, {}, options, function (err, out) {
@@ -379,8 +404,7 @@ suite('ejs.renderFile(path, [data], [options], fn)', function () {
 
   test('opts.context', function (done) {
     var ctxt = {foo: 'FOO'};
-    ejs.renderFile('test/fixtures/with-context.ejs', {},
-          {context: ctxt}, function(err, html) {
+    ejs.renderFile('test/fixtures/with-context.ejs', {}, {context: ctxt}, function(err, html) {
       if (err) {
         return done(err);
       }
@@ -389,14 +413,89 @@ suite('ejs.renderFile(path, [data], [options], fn)', function () {
     });
 
   });
+
+  test('support express multiple views folders, falls back to second if first is not available', function (done) {
+    var data = {
+      viewsText: 'test',
+      includePath: 'views-include.ejs',
+      settings: {
+        views: [
+          path.join(__dirname, 'fixtures/nonexistent-folder'),
+          path.join(__dirname, 'fixtures')
+        ]
+      }
+    };
+    ejs.renderFile(path.join(__dirname, 'fixtures/views.ejs'), data, function(error, data){
+      assert.ifError(error);
+      assert.equal('<div><p>global test</p>\n</div>\n', data);
+      done();
+    });
+
+  });
+
+  test('support express multiple views folders, falls back to second if first is not available (include preprocessor)', function (done) {
+    var data = {
+      viewsText: 'test',
+      settings: {
+        views: [
+          path.join(__dirname, 'fixtures/nonexistent-folder'),
+          path.join(__dirname, 'fixtures')
+        ]
+      }
+    };
+    ejs.renderFile(path.join(__dirname, 'fixtures/views-old.ejs'), data, function(error, data){
+      assert.ifError(error);
+      assert.equal('<div><p>global test</p>\n</div>\n', data);
+      done();
+    });
+
+  });
+
+  test('looks relative to the containing file first (include preprocessor)', function (done) {
+    var data = {
+      viewsText: 'test',
+      settings: {
+        views: [
+          path.join(__dirname, 'fixtures/views'),
+          path.join(__dirname, 'fixtures')
+        ]
+      }
+    };
+    ejs.renderFile(path.join(__dirname, 'fixtures/views-old.ejs'), data, function(error, data){
+      assert.ifError(error);
+      assert.equal('<div><p>global test</p>\n</div>\n', data);
+      done();
+    });
+
+  });
+
+  test('can reference by paths with directory names', function (done) {
+    var data = {
+      viewsText: 'test',
+      includePath: 'views/views-include.ejs',
+      settings: {
+        views: [
+          path.join(__dirname, 'fixtures/views'),
+          path.join(__dirname, 'fixtures')
+        ]
+      }
+    };
+    ejs.renderFile(path.join(__dirname, 'fixtures/views.ejs'), data, function(error, data){
+      assert.ifError(error);
+      assert.equal('<div><p>custom test</p>\n</div>\n', data);
+      done();
+    });
+
+  });
+
 });
 
 suite('cache specific', function () {
   test('`clearCache` work properly', function () {
-    var expected = '<p>Old</p>'
-      , file = __dirname + '/tmp/clearCache.ejs'
-      , options = {cache: true, filename: file}
-      , out = ejs.render('<p>Old</p>', {}, options);
+    var expected = '<p>Old</p>';
+    var file = __dirname + '/tmp/clearCache.ejs';
+    var options = {cache: true, filename: file};
+    var out = ejs.render('<p>Old</p>', {}, options);
     assert.equal(out, expected);
 
     ejs.clearCache();
@@ -407,11 +506,11 @@ suite('cache specific', function () {
   });
 
   test('`clearCache` work properly, LRU', function () {
-    var expected = '<p>Old</p>'
-      , oldCache = ejs.cache
-      , file = __dirname + '/tmp/clearCache.ejs'
-      , options = {cache: true, filename: file}
-      , out;
+    var expected = '<p>Old</p>';
+    var oldCache = ejs.cache;
+    var file = __dirname + '/tmp/clearCache.ejs';
+    var options = {cache: true, filename: file};
+    var out;
 
     ejs.cache = LRU();
 
@@ -426,11 +525,11 @@ suite('cache specific', function () {
   });
 
   test('LRU with cache-size 1', function () {
-    var oldCache = ejs.cache
-      , options
-      , out
-      , expected
-      , file;
+    var oldCache = ejs.cache;
+    var options;
+    var out;
+    var expected;
+    var file;
 
     ejs.cache = LRU(1);
 
@@ -476,6 +575,11 @@ suite('<%', function () {
 });
 
 suite('<%=', function () {
+  test('should not throw an error with a // comment on the final line', function () {
+    assert.equal(ejs.render('<%=\n// a comment\nname\n// another comment %>', {name: '&nbsp;<script>'}),
+      '&amp;nbsp;&lt;script&gt;');
+  });
+
   test('escape &amp;<script>', function () {
     assert.equal(ejs.render('<%= name %>', {name: '&nbsp;<script>'}),
         '&amp;nbsp;&lt;script&gt;');
@@ -505,6 +609,11 @@ suite('<%=', function () {
 });
 
 suite('<%-', function () {
+  test('should not throw an error with a // comment on the final line', function () {
+    assert.equal(ejs.render('<%-\n// a comment\nname\n// another comment %>', {name: '&nbsp;<script>'}),
+        '&nbsp;<script>');
+  });
+
   test('not escape', function () {
     assert.equal(ejs.render('<%- name %>', {name: '<script>'}),
         '<script>');
@@ -555,13 +664,13 @@ suite('-%>', function () {
   });
 
   test('works with unix style', function () {
-    var content = "<ul><% -%>\n"
-    + "<% users.forEach(function(user){ -%>\n"
-    + "<li><%= user.name -%></li>\n"
-    + "<% }) -%>\n"
-    + "</ul><% -%>\n";
+    var content = '<ul><% -%>\n'
+    + '<% users.forEach(function(user){ -%>\n'
+    + '<li><%= user.name -%></li>\n'
+    + '<% }) -%>\n'
+    + '</ul><% -%>\n';
 
-    var expectedResult = "<ul><li>geddy</li>\n<li>neil</li>\n<li>alex</li>\n</ul>";
+    var expectedResult = '<ul><li>geddy</li>\n<li>neil</li>\n<li>alex</li>\n</ul>';
     var fn;
     fn = ejs.compile(content);
     assert.equal(fn({users: users}),
@@ -569,13 +678,13 @@ suite('-%>', function () {
   });
 
   test('works with windows style', function () {
-    var content = "<ul><% -%>\r\n"
-    + "<% users.forEach(function(user){ -%>\r\n"
-    + "<li><%= user.name -%></li>\r\n"
-    + "<% }) -%>\r\n"
-    + "</ul><% -%>\r\n";
+    var content = '<ul><% -%>\r\n'
+    + '<% users.forEach(function(user){ -%>\r\n'
+    + '<li><%= user.name -%></li>\r\n'
+    + '<% }) -%>\r\n'
+    + '</ul><% -%>\r\n';
 
-    var expectedResult = "<ul><li>geddy</li>\r\n<li>neil</li>\r\n<li>alex</li>\r\n</ul>";
+    var expectedResult = '<ul><li>geddy</li>\r\n<li>neil</li>\r\n<li>alex</li>\r\n</ul>';
     var fn;
     fn = ejs.compile(content);
     assert.equal(fn({users: users}),
@@ -669,8 +778,8 @@ suite('exceptions', function () {
 
   var unhook = null;
   test('log JS source when debug is set', function (done) {
-    var out = ''
-      , needToExit = false;
+    var out = '';
+    var needToExit = false;
     unhook = hook_stdio(process.stdout, function (str) {
       out += str;
       if (needToExit) {
@@ -685,6 +794,22 @@ suite('exceptions', function () {
     });
     ejs.render(fixture('hello-world.ejs'), {}, {debug: true});
   });
+
+  test('escape filename in errors', function () {
+    assert.throws(function () {
+      ejs.render('<% throw new Error("whoops"); %>', {}, {filename: '<script>'});
+    }, /Error: &lt;script&gt;/);
+  });
+
+  test('filename in errors uses custom escape', function () {
+    assert.throws(function () {
+      ejs.render('<% throw new Error("whoops"); %>', {}, {
+        filename: '<script>',
+        escape: function () { return 'zooby'; }
+      });
+    }, /Error: zooby/);
+  });
+
   teardown(function() {
     if (!unhook) {
       return;
@@ -708,12 +833,24 @@ suite('include()', function () {
         fixture('include-simple.html'));
   });
 
+  test('include and escape ejs', function () {
+    var file = 'test/fixtures/include-escaped.ejs';
+    assert.equal(ejs.render(fixture('include-escaped.ejs'), {}, {filename: file}),
+        fixture('include-escaped.html'));
+  });
+
+  test('include in expression ejs', function () {
+    var file = 'test/fixtures/include-expression.ejs';
+    assert.equal(ejs.render(fixture('include-expression.ejs'), {}, {filename: file}),
+        fixture('include-expression.html'));
+  });
+
   test('include ejs fails without `filename`', function () {
     try {
       ejs.render(fixture('include-simple.ejs'));
     }
     catch (err) {
-      assert.ok(err.message.indexOf('requires the \'filename\' option') > -1);
+      assert.ok(err.message.indexOf('Could not find') > -1);
       return;
     }
     throw new Error('expected inclusion error');
@@ -741,11 +878,10 @@ suite('include()', function () {
   });
 
   test('include ejs with set root path', function () {
-    var file = 'test/fixtures/include-root.ejs',
-        viewsPath = path.join(__dirname, 'fixtures');
+    var file = 'test/fixtures/include-root.ejs';
+    var viewsPath = path.join(__dirname, 'fixtures');
     assert.equal(ejs.render(fixture('include-root.ejs'), {pets: users}, {filename: file, delimiter: '@',root:viewsPath}),
       fixture('include.html'));
-
   });
 
   test('work when nested', function () {
@@ -755,8 +891,8 @@ suite('include()', function () {
   });
 
   test('work with a variable path', function () {
-    var file = 'test/fixtures/menu_var.ejs',
-        includePath = 'includes/menu-item';
+    var file = 'test/fixtures/menu_var.ejs';
+    var includePath = 'includes/menu-item';
     assert.equal(ejs.render(fixture('menu.ejs'), {pets: users, varPath:  includePath}, {filename: file}),
       fixture('menu.html'));
   });
@@ -768,12 +904,12 @@ suite('include()', function () {
   });
 
   test('pass compileDebug to include', function () {
-    var file = 'test/fixtures/include.ejs'
-      , fn;
+    var file = 'test/fixtures/include.ejs';
+    var fn;
     fn = ejs.compile(fixture('include.ejs'), {
-      filename: file
-    , delimiter: '@'
-    , compileDebug: false
+      filename: file,
+      delimiter: '@',
+      compileDebug: false
     });
     try {
       // Render without a required variable reference
@@ -789,9 +925,9 @@ suite('include()', function () {
 
   test('is dynamic', function () {
     fs.writeFileSync(__dirname + '/tmp/include.ejs', '<p>Old</p>');
-    var file = 'test/fixtures/include_cache.ejs'
-      , options = {filename: file}
-      , out = ejs.compile(fixture('include_cache.ejs'), options);
+    var file = 'test/fixtures/include_cache.ejs';
+    var options = {filename: file};
+    var out = ejs.compile(fixture('include_cache.ejs'), options);
     assert.equal(out(), '<p>Old</p>\n');
 
     fs.writeFileSync(__dirname + '/tmp/include.ejs', '<p>New</p>');
@@ -800,10 +936,10 @@ suite('include()', function () {
 
   test('support caching', function () {
     fs.writeFileSync(__dirname + '/tmp/include.ejs', '<p>Old</p>');
-    var file = 'test/fixtures/include_cache.ejs'
-      , options = {cache: true, filename: file}
-      , out = ejs.render(fixture('include_cache.ejs'), {}, options)
-      , expected = fixture('include_cache.html');
+    var file = 'test/fixtures/include_cache.ejs';
+    var options = {cache: true, filename: file};
+    var out = ejs.render(fixture('include_cache.ejs'), {}, options);
+    var expected = fixture('include_cache.html');
     assert.equal(out, expected);
     out = ejs.render(fixture('include_cache.ejs'), {}, options);
     // No change, still in cache
@@ -813,6 +949,17 @@ suite('include()', function () {
     assert.equal(out, expected);
   });
 
+  test('handles errors in included file', function() {
+    try {
+      ejs.render('<%- include("fixtures/include-with-error") %>', {}, {filename: path.join(__dirname, 'f.ejs')});
+    }
+    catch (err) {
+      assert.ok(err.message.indexOf('foobar is not defined') > -1);
+      return;
+    }
+    throw new Error('expected inclusion error');
+  });
+
 });
 
 suite('preprocessor include', function () {
@@ -831,7 +978,7 @@ suite('preprocessor include', function () {
       ejs.render(fixture('include_preprocessor.ejs'), {pets: users}, {delimiter: '@'});
     }
     catch (err) {
-      assert.ok(err.message.indexOf('requires the \'filename\' option') > -1);
+      assert.ok(err.message.indexOf('Could not find') > -1);
       return;
     }
     throw new Error('expected inclusion error');
@@ -851,8 +998,8 @@ suite('preprocessor include', function () {
   });
 
   test('tracks dependency correctly', function () {
-    var file = 'test/fixtures/menu_preprocessor.ejs'
-      , fn = ejs.compile(fixture('menu_preprocessor.ejs'), {filename: file});
+    var file = 'test/fixtures/menu_preprocessor.ejs';
+    var fn = ejs.compile(fixture('menu_preprocessor.ejs'), {filename: file});
     assert(fn.dependencies.length);
   });
 
@@ -863,12 +1010,12 @@ suite('preprocessor include', function () {
   });
 
   test('pass compileDebug to include', function () {
-    var file = 'test/fixtures/include_preprocessor.ejs'
-      , fn;
+    var file = 'test/fixtures/include_preprocessor.ejs';
+    var fn;
     fn = ejs.compile(fixture('include_preprocessor.ejs'), {
-      filename: file
-    , delimiter: '@'
-    , compileDebug: false
+      filename: file,
+      delimiter: '@',
+      compileDebug: false
     });
     try {
       // Render without a required variable reference
@@ -884,9 +1031,9 @@ suite('preprocessor include', function () {
 
   test('is static', function () {
     fs.writeFileSync(__dirname + '/tmp/include_preprocessor.ejs', '<p>Old</p>');
-    var file = 'test/fixtures/include_preprocessor_cache.ejs'
-      , options = {filename: file}
-      , out = ejs.compile(fixture('include_preprocessor_cache.ejs'), options);
+    var file = 'test/fixtures/include_preprocessor_cache.ejs';
+    var options = {filename: file};
+    var out = ejs.compile(fixture('include_preprocessor_cache.ejs'), options);
     assert.equal(out(), '<p>Old</p>\n');
 
     fs.writeFileSync(__dirname + '/tmp/include_preprocessor.ejs', '<p>New</p>');
@@ -895,10 +1042,10 @@ suite('preprocessor include', function () {
 
   test('support caching', function () {
     fs.writeFileSync(__dirname + '/tmp/include_preprocessor.ejs', '<p>Old</p>');
-    var file = 'test/fixtures/include_preprocessor_cache.ejs'
-      , options = {cache: true, filename: file}
-      , out = ejs.render(fixture('include_preprocessor_cache.ejs'), {}, options)
-      , expected = fixture('include_preprocessor_cache.html');
+    var file = 'test/fixtures/include_preprocessor_cache.ejs';
+    var options = {cache: true, filename: file};
+    var out = ejs.render(fixture('include_preprocessor_cache.ejs'), {}, options);
+    var expected = fixture('include_preprocessor_cache.html');
     assert.equal(out, expected);
     fs.writeFileSync(__dirname + '/tmp/include_preprocessor.ejs', '<p>New</p>');
     out = ejs.render(fixture('include_preprocessor_cache.ejs'), {}, options);
@@ -906,13 +1053,24 @@ suite('preprocessor include', function () {
   });
 
   test('whitespace slurp and rmWhitespace work', function() {
-    var file = 'test/fixtures/include_preprocessor_line_slurp.ejs'
-      , template = fixture('include_preprocessor_line_slurp.ejs')
-      , expected = fixture('include_preprocessor_line_slurp.html')
-      , options = {rmWhitespace: true, filename: file};
-    assert.equal(ejs.render(template, options),
+    var file = 'test/fixtures/include_preprocessor_line_slurp.ejs';
+    var template = fixture('include_preprocessor_line_slurp.ejs');
+    var expected = fixture('include_preprocessor_line_slurp.html');
+    var options = {rmWhitespace: true, filename: file};
+    assert.equal(ejs.render(template, {}, options),
         expected);
-  })
+  });
+
+  test('handles errors in included file', function() {
+    try {
+      ejs.render('<%- include fixtures/include-with-error %>', {}, {filename: path.join(__dirname, 'f.ejs')});
+    }
+    catch (err) {
+      assert.ok(err.message.indexOf('foobar is not defined') > -1);
+      return;
+    }
+    throw new Error('expected inclusion error');
+  });
 
 });
 
@@ -927,12 +1085,31 @@ suite('require', function () {
 
   // Only works with inline/preprocessor includes
   test('allow ejs templates to be required as node modules', function () {
-      var file = 'test/fixtures/include_preprocessor.ejs'
-        , template = require(__dirname + '/fixtures/menu_preprocessor.ejs');
-      if (!process.env.running_under_istanbul) {
-        assert.equal(template({filename: file, pets: users}),
+    var file = 'test/fixtures/include_preprocessor.ejs';
+    var template = require(__dirname + '/fixtures/menu_preprocessor.ejs');
+    if (!process.env.running_under_istanbul) {
+      assert.equal(template({filename: file, pets: users}),
           fixture('menu_preprocessor.html'));
+    }
+  });
+});
+
+suite('test fileloader', function () {
+
+  var myFileLoad = function (filePath) {
+    return 'myFileLoad: ' + fs.readFileSync(filePath);
+  };
+
+  test('test custom fileload', function (done) {
+    ejs.fileLoader = myFileLoad;
+    ejs.renderFile('test/fixtures/para.ejs', function(err, html) {
+      if (err) {
+        return done(err);
       }
+      assert.equal(html, 'myFileLoad: <p>hey</p>\n');
+      done();
+    });
+
   });
 });
 
@@ -944,8 +1121,8 @@ suite('examples', function () {
     }
     suite(f, function () {
       test('doesn\'t throw any errors', function () {
-        var stderr = hook_stdio(process.stderr, noop)
-          , stdout = hook_stdio(process.stdout, noop);
+        var stderr = hook_stdio(process.stderr, noop);
+        var stdout = hook_stdio(process.stdout, noop);
         try {
           require('../examples/' + f);
         }
@@ -960,3 +1137,13 @@ suite('examples', function () {
     });
   });
 });
+
+suite('meta information', function () {
+  test('has a version', function () {
+    assert.strictEqual(ejs.VERSION, require('../package.json').version);
+  });
+
+  test('had a name', function () {
+    assert.strictEqual(ejs.name, 'ejs');
+  });
+});
